@@ -1,6 +1,47 @@
 
 use crate::tachyon::*;
 
+use super::pool::Pool;
+
+#[no_mangle]
+pub extern "C" fn null_test() -> *mut Pool {
+    return std::ptr::null_mut();
+}
+
+#[no_mangle]
+pub extern "C" fn create_pool() -> *mut Pool {
+    let pool = Pool::create();
+    let b = Box::new(pool);
+    return Box::into_raw(b);
+}
+
+#[no_mangle]
+pub extern "C" fn destroy_pool(pool: *mut Pool) {
+    if !pool.is_null() {
+        let _b = unsafe {Box::from_raw(pool)};
+    }
+}
+
+#[no_mangle]
+pub extern "C" fn create_pool_server(pool_ptr: *mut Pool, config_ptr: *const TachyonConfig, naddress: *const NetworkAddress) -> *mut Tachyon {
+    let pool = unsafe {&mut*pool_ptr};
+    let config: TachyonConfig = unsafe { std::ptr::read(config_ptr as *const _) };
+    let address: NetworkAddress = unsafe { std::ptr::read(naddress as *const _) };
+    match pool.create_server(config, address) {
+        Some(server) => {return server;},
+        None => return std::ptr::null_mut(),
+    }
+}
+
+#[no_mangle]
+pub extern "C" fn get_pool_server(pool_ptr: *mut Pool, id: u16) -> *mut Tachyon {
+    let pool = unsafe {&mut*pool_ptr};
+    
+    match pool.get_server(id) {
+        Some(server) => {return server;},
+        None => return std::ptr::null_mut(),
+    }
+}
 
 #[no_mangle]
 pub extern "C" fn create_tachyon(config_ptr: *const TachyonConfig) -> *mut Tachyon {
@@ -21,20 +62,31 @@ pub extern "C" fn destroy_tachyon(tachyon: *mut Tachyon) {
 pub extern "C" fn bind_socket(tachyon_ptr: *mut Tachyon, naddress: *const NetworkAddress) -> i32 {
     let tachyon = unsafe {&mut*tachyon_ptr};
     let address: NetworkAddress = unsafe { std::ptr::read(naddress as *const _) };
-    return tachyon.bind(address);
+    match tachyon.bind(address) {
+        true => return 1,
+        false => return -1,
+    }
 }
 
 #[no_mangle]
 pub extern "C" fn connect_socket(tachyon_ptr: *mut Tachyon, naddress: *const NetworkAddress) -> i32 {
     let tachyon = unsafe {&mut*tachyon_ptr};
     let address: NetworkAddress = unsafe { std::ptr::read(naddress as *const _) };
-    return tachyon.connect(address);
+    match tachyon.connect(address) {
+        true => return 1,
+        false => return -1,
+    }
 }
 
 #[no_mangle]
-pub extern "C" fn configure_channel(tachyon_ptr: *mut Tachyon, channel_id: u8, ordered: u8) {
+pub extern "C" fn configure_channel(tachyon_ptr: *mut Tachyon, channel_id: u8, ordered: u8) -> i32 {
     let tachyon = unsafe {&mut*tachyon_ptr};
-    tachyon.configure_channel(channel_id, ordered == 1);
+    let res = tachyon.configure_channel(channel_id, ordered == 1);
+    if res {
+        return 1;
+    } else {
+        return -1;
+    }
 }
 
 fn copy_send_result(from: TachyonSendResult, to: *mut TachyonSendResult) {
@@ -107,10 +159,10 @@ pub extern "C" fn tachyon_update(tachyon_ptr: *mut Tachyon) {
 }
 
 #[no_mangle]
-pub extern "C" fn get_client_addresses(tachyon_ptr: *mut Tachyon, addresses: *mut NetworkAddress, max: u16) -> u16 {
+pub extern "C" fn get_connections(tachyon_ptr: *mut Tachyon, addresses: *mut Connection, max: u16) -> u16 {
     let tachyon = unsafe {&mut*tachyon_ptr};
     
-    let list = tachyon.get_address_list(max);
+    let list = tachyon.get_connections(max);
     
     unsafe {
         std::ptr::copy_nonoverlapping(list.as_ptr(), addresses, list.len());
